**free                                                                                              
                                                                                                    
// RPG BATTLE DEV 2023 - Manche 1 - Epreuve 2 - Du 7 au 9 juin 2023                                 
                                                                                                    
// Contexte :                                                                                       
//  Définition : un palindrome est un mot qui peut se lire dans les 2 sens. Exemple : radar         
//  Le fichier base de données TEAMLIB/MOTS contient une liste de mots (1 mot par enregistrement)   
                                                                                                    
// Objectif :                                                                                       
//  Afficher le nombre de palindromes présents et les afficher (DSPLY) par ordre alphabétique       
//  Exemple de résultat :                                                                           
//   DSPLY 2                                                                                        
//   DSPLY abcba                                                                                    
//   DSPLY radar                                                                                    
                                                                                                    
// Consignes :                                                                                      
//  1. Ne pas modifier les lignes de code déjà présentes, ajouter vos lignes de code nécessaires    
//  2. Utiliser uniquement des instructions RPG natives (pas de SQL, pas d'appels d'API ...)        
//  3. Dans le fichier MOTS :                                                                       
//     - Ne pas tenir compte des espaces et des ponctuations                                        
//     - Considérer les caractères accentués comme leurs caractères de base (é => e, ç => c)        
//  4. En cas de doublon, n'afficher qu'une seule occurrence du mot                                 
                                                                                                    
// Critères d'évaluation :                                                                          
//  1. Le résultat doit être celui demandé                                                          
//  2. Le code présent ne doit pas être modifié                                                     
//  3. Le code doit être le plus simple, le plus lisible possible et le plus optimisé               
//  4. Le programme doit s'appeler E2 dans votre bibliothèque TEAMxx                                
                                                                                                    
// ------------------------------------------------------------------------------------------------ 
// Votre profil IBM i : TEAMxx                                                                      
// ------------------------------------------------------------------------------------------------ 
                                                                                                    
                                                                                                    
ctl-opt actgrp(*new) alwnull(*no);                                                                  
ctl-opt srtseq(*langidshr) altseq(*ext);                                                            
                                                                                                    
dcl-f mots rename(mots:fmots);                                                                      
                                                                                                    
// Stockage des palindromes : originaux et "normalisés" (sans espaces, caractères accentués)        
dcl-ds palindrome qualified ;                                                                       
  origine   like(MOT) inz ;                                                                         
  normalise like(MOT) inz ;                                                                         
end-ds ;                                                                                            
dcl-ds palindromeListe likeds(palindrome) dim(*auto:200) ;                                          
dcl-s normalise like(MOT) inz ;                                                                     
                                                                                                    
                                                                                                    
// Parcours du fichier                                                                              
read fmots ;                                                                                        
dow not %eof( mots ) ;                                                                              
                                                                                                    
  // Suppression des blancs, passage en minuscule, suppression des accents usuels et ponctuations   
  normalise = normaliser( MOT ) ;                                                                   
  // on traite si pas déjà présent dans la liste et si c'est un palindrome                          
  if %lookup( normalise : palindromeListe(*).normalise ) = 0 and                                    
    estPalindrome(normalise) ;                                                                      
       palindromeListe(*next).origine  = MOT ;                          // pour affichage original  
       palindromeListe(%elem(palindromeListe)).normalise = normalise ;  // pour recherche           
  endif ;                                                                                           
                                                                                                    
  // enregistrement suivant                                                                         
  read fmots ;                                                                                      
enddo ;                                                                                             
                                                                                                    
// Affichage :                                                                                      
afficher() ;                                                                                        
                                                                                                    
return ;                                                                                            
                                                                                                    
                                                                                                    
// Détermine si une valeur est un palindrome ?                                                      
dcl-proc estPalindrome ;                                                                            
  dcl-pi *n ind ;                                                                                   
    p_valeur like(MOT) const ;                                                                      
  end-pi;                                                                                           
                                                                                                    
  dcl-s min uns(3) inz(1) ;                                                                         
  dcl-s max uns(3) inz    ;                                                                         
                                                                                                    
  // On parcours en comparant début et fin tant que égalité                                         
  max = %len(%trim( p_valeur )) ;                                                                   
  dow min < max and                                                                                 
      %subst( p_valeur : min : 1 ) = %subst( p_valeur : max : 1 ) ;                                 
    min += 1 ;                                                                                      
    max -= 1 ;                                                                                      
  enddo ;                                                                                           
                                                                                                    
  return %subst( p_valeur : min : 1 ) = %subst( p_valeur : max : 1 ) ;                              
                                                                                                    
end-proc;                                                                                           
                                                                                                    
                                                                                                    
// Tri naturel et affichage                                                                         
dcl-proc afficher ;                                                                                 
  sorta palindromeListe(*).origine ;                                                                
  dsply %elem( palindromeListe ) ;                                                                  
  for-each palindrome in palindromeListe ;                                                          
    dsply palindrome.origine ;                                                                      
  endfor;                                                                                           
end-proc ;                                                                                          
                                                                                                    
                                                                                                    
// Suppression des accents et de tous les caractères non alphabétiques                              
dcl-proc normaliser ;                                                                               
  dcl-pi *n like( MOT ) ;                                                                           
    p_MOT like( MOT ) const ;                                                                       
  end-pi ;                                                                                          
                                                                                                    
  dcl-s l_MOT        like( MOT ) inz ;                                                              
  dcl-s l_ASupprimer like( MOT ) inz ;                                                              
  dcl-c ALPHABET     const('abcdefghijklmnopqrstuvwxyz0123456789') ;                                
  dcl-c BLANC        const('                                    ') ;                                
                                                                                                    
  // Remplacement des accents (à complèter), passage en minuscule                                   
  l_MOT = %xlate( 'éèêëàäâçïîöôüûù' : 'eeeeaaaciioouuu' : %lower( p_MOT ) ) ;                       
  // On supprime toutes les lettres : il restera ce que l'on doit supprimer                         
  l_ASupprimer = %xlate( ALPHABET : BLANC : l_MOT ) ;                                               
  // Suppression des caractères non alpha                                                           
  l_MOT = %scanrpl( ' ' : '' :                                                                      
                    %xlate( l_ASupprimer : %subst( BLANC : 1 : %len(l_ASupprimer) ) : l_MOT ) ) ;   
  return l_MOT ;                                                                                    
end-proc ;                                                                                          
