**free

// RPG BATTLE DEV 2023 - Manche 2 - Epreuve 4 - Du 14 au 16 juin 2023

// Contexte :
//  - Un fichier CSV contient des données séparées par le caractère ;
//  - Un fichier JSON contient des données représentées sous forme de paires clés/valeurs
//    contenues dans un objet : {"code": "valeur du code", "prix": 12}
//  - Un tableau JSON est matérialisé par [ valeur1, valeur2... ] valeurx étant un objet JSON
//    Exemple : [{"p1": "peinture bleue", "prix": 10}, {"p2": "peinture verte", "prix": 9.5}]

// Objectif :
//  Depuis le fichier base de données (TEAMLIB/INFOS) contenant des données au format CSV,
//  construire la valeur JSON correspondante dans la variable JSONVAR

// Exemple :
// - Le fichier INFOS fourni contient les lignes suivantes :
//    nom;prenom;compagnie
//    Gates;Bill;Microsoft
//    Soltis;Frank;IBM
//    Jobs;Steve;Apple
// - Vous devez construire la chaîne JSON suivante :
//   JSONVAR => [{"nom": "Gates", "prenom": "Bill", "compagnie": "Microsoft"},
//               {"nom": "Soltis", "prenom": "Frank", "compagnie": "IBM"},
//               {"nom": "Jobs", "prenom": "Steve", "compagnie": "Apple"}]
// - Envoyez la valeur de JSONVAR en tant que message dans la JOBLOG (SND-MSG)

// Consignes :
//  1. Ne pas modifier les lignes de code déjà présentes, ajouter vos lignes de code nécessaires
//  2. Utiliser uniquement des instructions RPG natives (pas de SQL, pas d'appels d'API ...)
//  3. Lorsqu'une valeur n'est pas alimentée dans le fichier INFOS (nom, prénom ou compagnie),
//     indiquer : "*** Inconnu ***"
//  4. Certains caractères nécessitent un traitement spécial en JSON : cf https://www.json.org/json-

// Critères d'évaluation :
//  1. Le résultat doit être celui demandé
//  2. Le code présent ne doit pas être modifié
//  3. Le code doit être le plus simple, le plus lisible et le plus optimisé
//  4. Le programme doit s'appeler E4 dans votre bibliothèque TEAMxx


// ------------------------------------------------------------------------------------------------
// Votre profil IBM i : TEAMxx
// ------------------------------------------------------------------------------------------------

ctl-opt actgrp(*new) alwnull(*no);

dcl-f infos ;
dcl-s JSONVAR varchar(100000) inz ;

// Nombre maximum de colonnes admises
dcl-c MAX_COL const(50) ;
// Taille maximale d'une valeur admise
dcl-c MAX_LEN const(50) ;
// Parsing des entêtes
dcl-s entetes varchar(MAX_LEN) dim(*auto : MAX_COL) inz ;
// n° de ligne lue
dcl-s numLigne uns(10) inz(0) ;


// Lecture de l'entête (ligne 1)
// On admet que l'ensemble des colonnes est ici alimenté
read infof ;
entetes = %split( ligne : ';' ) ;

// Lecture du contenu du fichier
read infof ;
dow not %eof( infos ) ;
  numLigne += 1 ;                         // N° de ligne
  JSONVAR += ajouter( ligne :numLigne ) ; // ne pas transmettre entetes en paramètre :
                                          // non supporté pour *auto
  // ligne suivante
  read infof ;
enddo ;

// Formatage tableau JSON et affichage
SND-MSG *INFO '[' + JSONVAR + ']' ;

return ;



// construction d'un objet JSON correspondant à 1 ligne de csv
dcl-proc ajouter ;
  dcl-pi *n varchar(2000);
    p_valeur like( LIGNE ) options(*trim) const ;
    p_numLigne like(numLigne) const ;
  end-pi ;
  // variables
  dcl-s objet       varchar(2000) inz ;
  dcl-s cpt         uns(3) inz(1) ;
  dcl-s valeurs     varchar(MAX_LEN) dim(*auto : MAX_COL) ;

  // Découpage des valeurs
  valeurs = %split( p_valeur : ';' ) ;
  // on limite ou augmente pour être cohérent avec le nombre d'entêtes
  %elem(valeurs) = %elem(entetes) ;

  // Traitement des valeurs
  for cpt = 1 to %elem(entetes) ;
    if cpt > 1 ;
      objet += ', ' ;
    endif ;
    if valeurs(cpt) = '' ;
      valeurs(cpt) = '"*** Inconnu ***"' ;
    endif ;
    // Echappement des caractères " dans la valeur et constitution objet
    objet += '"' + %trim(entetes(cpt)) + '": ' +
             '"' + %trim( %scanrpl( '"' : '\"' : valeurs(cpt) ) ) + '"' ;
  endfor ;

  if p_numLigne = 1 ;
    return '{' + objet + '}' ;
  endif ;
  return ', {' + objet + '}' ;
end-proc ;
